﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;

// Throat Bulger Plugin by Captain Varghoss
//
// Original plugin code by VRAdultFun
//
// 0.1 Fixed target atom selection to have scrollable popup - Captain Varghoss
// 0.2 Added additional morph increasing from 2 to 3 and adjusted calculations to compensate. - Captain Varghoss
// 1.0 Complete rewrite. All new custom morphs, increasing from 3 to 8 for more detailed bulging. - Captain Varghoss
//     Split deep throat functions into separate plugin for flexibility with multiple atoms and instances. 
//
//
// Shoutout to Juno for his tutorial and input with making the morphs, and thanks to VRAdultFun for the original plugin and idea.
//

namespace CaptainVarghoss {
	public class ThroatBulger : MVRScript {
		protected JSONStorableBool isActive;
		protected JSONStorableStringChooser uiFocusTarget;
		protected JSONStorableFloat uiBulgeMult;
		protected JSONStorableFloat uiMinBulgeDistMult;
		protected JSONStorableFloat uiBulge1Mult;
		protected JSONStorableFloat uiBulge2Mult;
		protected JSONStorableFloat uiBulge3Mult;

		protected JSONStorableFloat uiToyMulti;
		protected JSONStorableFloat uiCurrentDepth;
		protected UIDynamicTextField helpTextfield;
		protected JSONStorableString helpText;
		protected UIDynamicTextField helpTextfield2;
		protected JSONStorableString helpText2;
		protected JSONStorableString stateInfos;

		private static float toyFloat = 0.0f;
		private static float tempFloat = 0.0f;
		
        private static FreeControllerV3 headController;
		private static FreeControllerV3 pelvisController;
        private static DAZMorph morphDeepBulgeThroat1;
        private static DAZMorph morphDeepBulgeThroat2;
		private static DAZMorph morphDeepBulgeThroat3;
		
        private static Atom person;
        private static Atom person2;
		private static Atom currentAtom;
		private static string currentAtomName = "None";
		private static FreeControllerV3 playerPelvisController;
        private static FreeControllerV3 playerHeadController;
        private static FreeControllerV3 playerTipController;
        private static FreeControllerV3 playerTipBaseController;
		private static bool isObject = false;

		public override void Init() {
			try {
				// Active Toggle
				isActive = new JSONStorableBool ("isActive", true);
				RegisterBool (isActive);
				var toggle = CreateToggle(isActive);
				toggle.label = "Active";
				
				JSONStorableAction ToggleActive = new JSONStorableAction("Toggle Throat Bulger", () =>
                {
                    isActive.val = !isActive.val;
                });
                RegisterAction(ToggleActive);
				List<string> targetChoices = new List<string>();
				foreach (string atomUID in SuperController.singleton.GetAtomUIDs())
				{
					currentAtom = SuperController.singleton.GetAtomByUid(atomUID);
					if (currentAtom != containingAtom && atomUID != null)
					{
						targetChoices.Add(atomUID);
					}
				}
				uiFocusTarget = new JSONStorableStringChooser("Sex Giver", targetChoices, "None", "Choose Person");
				RegisterStringChooser(uiFocusTarget);
				UIDynamicPopup udp = CreateScrollablePopup(uiFocusTarget, true);
				
				uiBulgeMult = new JSONStorableFloat("Throat Bulge Multiplier", 0.50f, 0.0f, 1.0f, true, true);
				RegisterFloat(uiBulgeMult);
				CreateSlider(uiBulgeMult, false);
				
				uiMinBulgeDistMult = new JSONStorableFloat("Min Bulge Dist Multiplier", 0.50f, 0.0f, 5.0f, true, true);
				RegisterFloat(uiMinBulgeDistMult);
				CreateSlider(uiMinBulgeDistMult, false);
				
				uiToyMulti = new JSONStorableFloat("Toy Distance Multiplier", 5.0f, 0.0f, 5.0f, true, true);
				RegisterFloat(uiToyMulti);
				
				uiCurrentDepth = new JSONStorableFloat("Current Depth", 0.0f, -1.0f, 1.0f, true, true);
				RegisterFloat(uiCurrentDepth);
				CreateSlider(uiCurrentDepth);
				
				var btn = CreateButton("Reset Morphs", false);
                btn.button.onClick.AddListener(() => { ResetMorphs(); });
				
				helpText = new JSONStorableString("Helptext", "\n<b>Throat Bulger</b>\n\nChoose person: Select the atom that is performing the insertion, person and toy atoms work.\n\nSet Throat Bulge Multiplier to change how much of the morphs are used\n\nSet Min Bulge Dist Multiplier to fake actual depth, this helps with making the insertion appear much deeper than possible due to collisions.\n\nIf morphs get stuck for some reason, use the Reset Morphs button or toggle the active checkbox.");
				UIDynamic helpTextfield = CreateTextField(helpText, true);
				helpTextfield.height = 700f;

				// Per morph multiplier reducers
				helpText2 = new JSONStorableString("Helptext2", "\n<b>Bulge Morph Adjustments</b>\n\nUse these morphs to reduce Bulge Multiplier on individual morphs.\nThese can help on male/futa models that morph oddly. Also helps with the first morph being \"too much\".");
				UIDynamic helpTextfield2 = CreateTextField(helpText2, false);
				helpTextfield2.height = 300f;
				
				uiBulge1Mult = new JSONStorableFloat("Bulge 1 Amount Mult", 1.0f, 0.0f, 1.0f, true, true);
				RegisterFloat(uiBulge1Mult);
				CreateSlider(uiBulge1Mult, false);

				uiBulge2Mult = new JSONStorableFloat("Bulge 2 Amount Mult", 1.0f, 0.0f, 1.0f, true, true);
				RegisterFloat(uiBulge2Mult);
				CreateSlider(uiBulge2Mult, false);

				uiBulge3Mult = new JSONStorableFloat("Bulge 3 Amount Mult", 1.0f, 0.0f, 1.0f, true, true);
				RegisterFloat(uiBulge3Mult);
				CreateSlider(uiBulge3Mult, false);
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		void Start() {
			try {
				//SuperController.LogError("Start called");
				person = containingAtom;
				if (person != null)
				{
					headController = person.GetStorableByID("headControl") as FreeControllerV3;
					pelvisController = person.GetStorableByID("hipControl") as FreeControllerV3;
					ResetMorphs();
					
					if (uiFocusTarget.val != "none")
					{
						person2 = SuperController.singleton.GetAtomByUid(uiFocusTarget.val);
						if (person2 != null)
						{
							playerPelvisController = person2.GetStorableByID("pelvisControl") as FreeControllerV3;
						}
					}
				}
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}
		
		private void ResetMorphs() {
			//SuperController.LogError("ResetMorphs called");
			JSONStorable js = person.GetStorableByID("geometry");
			DAZCharacterSelector dcs = js as DAZCharacterSelector;
			GenerateDAZMorphsControlUI morphUI = dcs.morphsControlUI;
			
			if (morphUI != null) {
				//SuperController.LogError("ResetMorphs morphUI");
				morphDeepBulgeThroat1 = morphUI.GetMorphByDisplayName("Throat Bulge 1");
				morphDeepBulgeThroat1.Reset();
				morphDeepBulgeThroat2 = morphUI.GetMorphByDisplayName("Throat Bulge 2");
				morphDeepBulgeThroat2.Reset();
				morphDeepBulgeThroat3 = morphUI.GetMorphByDisplayName("Throat Bulge 3");
				morphDeepBulgeThroat3.Reset();
			}
		}
		
		void FixedUpdate() {
			if (isActive.val == true) {
				//SuperController.LogError("FixedUpdate Called");
				try {
					currentAtomName = uiFocusTarget.val;
					if ((person2 == null || playerTipController == null) && currentAtomName != "None")
					{
						person2 = SuperController.singleton.GetAtomByUid(uiFocusTarget.val);
						if (person2 == null)
						{
							uiFocusTarget.val = "None";
							currentAtomName = "None";
						}
						else
						{
							if (person2.type == "Person")
							{
								isObject = false;
								toyFloat = 0.0f;
								playerTipController = person2.GetStorableByID("penisTipControl") as FreeControllerV3;
							}
							else
							{
								isObject = true;
								toyFloat = uiToyMulti.val;
								playerTipController = person2.GetStorableByID("control") as FreeControllerV3;
							}
						}
					}
					//SuperController.LogError("FixedUpdate Called: " + morphDeepBulgeThroat2);
					if (morphDeepBulgeThroat1 != null && morphDeepBulgeThroat2 != null && morphDeepBulgeThroat3 != null && person2 != null && playerTipController != null)
					{
						tempFloat = Vector3.Distance(headController.followWhenOff.position, playerTipController.followWhenOff.position);
						//SuperController.LogError("Updated temp float " + tempFloat);

						// Morph 1
						if ( tempFloat < 0.15f * uiMinBulgeDistMult.val)
						{	
							morphDeepBulgeThroat1.morphValue = Mathf.Clamp((1.0f - (tempFloat*(6.666f / uiMinBulgeDistMult.val))) * (uiBulgeMult.val * uiBulge1Mult.val),0.0f,1.0f);
						}
						else
						{
							morphDeepBulgeThroat1.Reset();
						}
						// Morph 2
						if ( tempFloat < 0.12f * uiMinBulgeDistMult.val)
						{
							morphDeepBulgeThroat2.morphValue = Mathf.Clamp((1.0f - (tempFloat*(16f / uiMinBulgeDistMult.val))) * (uiBulgeMult.val * uiBulge2Mult.val),0.0f,1.0f);
						}
						else
						{
							morphDeepBulgeThroat2.Reset();
						}
						// Morph 3
						if ( tempFloat < 0.7f * uiMinBulgeDistMult.val)
						{
							morphDeepBulgeThroat3.morphValue = Mathf.Clamp((1.0f - (tempFloat*(22f / uiMinBulgeDistMult.val))) * (uiBulgeMult.val * uiBulge3Mult.val),0.0f,1.0f);
						}
						else
						{
							morphDeepBulgeThroat3.Reset();
						}
					}
					
					uiCurrentDepth.val = tempFloat;
				}
				catch (Exception e) {
					SuperController.LogError("Exception caught: " + e);
				}
			} else { ResetMorphs(); }
		}

		void OnDestroy() {
		}

	}
}